#!/usr/bin/perl

# shuffle.pl
# IXNvg

require 'config.txt';
require 'random.pl';

# ubN
$block_num = $#block_name + 1;

# C
sub shuffle_main {
	my @characters = &read_characters();
	@characters = &shuffle_characters(@characters);

	my $bone = &get_brackets_bone($#characters + 1);

	&output_brackets($bone, @characters);
}

# LN^[ǂݍ
sub read_characters {
	open(CH, $path_characters) || die "Failed to open: $path_characters\n";
	my @characters = <CH>;
	close(CH);

	printf STDERR "Character Count: %d\n", $#characters + 1;

	return @characters;
}

# LN^[Vbt
sub shuffle_characters {
	my (@characters) = @_;

	my $char_num = $#characters + 1;

	for (my $i = 0; $i < $char_num - 1; $i++) {
		my $j = int($rand_gen->rand($char_num - $i));
		if ($i != $j) {
			$_ = $characters[$i];
			$characters[$i] = $characters[$j];
			$characters[$j] = $_;
		}
	}

	return @characters;
}

# ΐ\̍iieƂ̐lj擾
#   $char_num : l
sub get_brackets_bone {
	my ($char_num) = @_;

	# ΐ\̍i
	my $bone = [];

	# LȂ
	if ($char_num <= 0) {
		print STDERR "No characters.\n";
		exit 1;
	}

	my $match_num = $match_per_block * $block_num;
	my $base_char_num = $char_per_match * $match_num;

	my $frac_num  = $char_num - $base_char_num;

	# LȂ
	if ($frac_num < 0) {
		print STDERR "Too few characters.\n";
		exit 1;
	}

	# l
	my $actual_char_per_match = $char_per_match;
	while ($frac_num > $match_num) {
		$frac_num -= $match_num;
		$actual_char_per_match++;
	}

	# ftHg̍i쐬
	for (my $i = 0; $i < $block_num; $i++) {
		my @tmp = ();
		for (my $j = 0; $j < $match_per_block; $j++) {
			push(@tmp, $actual_char_per_match);
		}
		push(@$bone, \@tmp);
	}

	if ($frac_num != 0) {
		if ($frac_treat == 0) {
			$bone = &add_frac0($frac_num, $bone);
		} elsif ($frac_treat == 1) {
			$bone = &add_frac1($frac_num, $bone);
		} else {
			$bone = &add_frac2($frac_num, $bone);
		}
	}

	return $bone;
}

# SubNɐ擪ubN珇ɔzuiA, B, C, ...j
sub add_frac0 {
	my ($frac_num, $bone) = @_;

	return $bone if ($frac_num == 0);

	for (my $f = 0; $f < $frac_num; $f++) {
		my $i = $f % $block_num;
		my $j = int($f / $block_num);

		$bone->[$i]->[$j]++;
	}

	return $bone;
}

# SubNɃ_ɕU
sub add_frac1 {
	my ($frac_num, $bone) = @_;

	return $bone if ($frac_num == 0);

	my @block_order = ();
	my @match_order = ();

	for (my $i = 0; $i < $block_num; $i++) {
		@match_order[$i] = &get_order2($match_per_block);
	}

	for (my $f = 0; $f < $frac_num; $f++) {
		my $oi = $f % $block_num;
		my $oj = int($f / $block_num);
		if ($oi == 0) {
			@block_order = &get_order2($block_num);
		}
		my $i = $block_order[$oi];

		$bone->[$i]->[$match_order[$i][$oj]]++;
	}

	return $bone;
}

sub get_order2 {
	my ($size) = @_;

	my @order = 0..($size-1);

	for (my $i = 0; $i < $size - 1; $i++) {
		my $j = int($rand_gen->rand($size - $i));
		if ($i != $j) {
			$_ = $order[$i];
			$order[$i] = $order[$j];
			$order[$j] = $_;
		}
	}

	return @order;
}

# 擪ubN珇ɋlߍށiA, A, A, ...j
sub add_frac2 {
	my ($frac_num, $bone) = @_;

	return $bone if ($frac_num == 0);

	for (my $f = 0; $f < $frac_num; $f++) {
		my $i = int($f / $match_per_block);
		my $j = $f % $match_per_block;

		$bone->[$i]->[$j]++;
	}

	return $bone;
}

# ΐ\o
sub output_brackets {
	my ($bone, @characters) = @_;

	open(BRA, ">$path_brackets") || die "Failed to open: $path_brackets\n";

	my $c = 0;

	for (my $i = 0; $i < $block_num; $i++) {
		for (my $j = 0; $j < $match_per_block; $j++) {
			my $n = $bone->[$i]->[$j];
			for (my $k = 0; $k < $n; $k++) {
				if ($c > $#characters + 1) {
					printf STDERR "Internal Error: Size mismatched.\n";
					exit 1;
				}
				printf BRA $output_format, $block_name[$i], $j + 1, $characters[$c];
				$c++;
			}
		}
	}

	if ($c != $#characters + 1) {
		printf STDERR "Internal Error: Size mismatched.\n";
		exit 1;
	}

	close(BRA);
}

# Cs
&shuffle_main;

1;
